----------------------------------------------------------------------------------
-- Company: FRI 
-- Engineer: Miha Moskon
-- 
-- Create Date:    10:49:51 09/28/2007 
-- Design Name: 
-- Module Name:    lcd_top - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity lcd_module is
	port (
		CLK_in	: in	STD_LOGIC;		-- clock
		RST_in	: in	STD_LOGIC;		-- reset
		
		DATA_RDY_in	: in	STD_LOGIC;	-- podatek pripravljen
		LCD_READY_out	: out	STD_LOGIC;	-- pripravljen za sprejem podatka - dokler je LCD_READY = '0' moram drzati stabilen podatek DATA_TO_SEND
		DATA_TO_SEND_in	: in	STD_LOGIC_VECTOR(9 downto 0);		-- podatki, ki jih posiljam LCD-ju
				
		LCD_E_out	: out	STD_LOGIC;		-- lcd enable
		LCD_RS_out	: out	STD_LOGIC;		-- lcd register select
		LCD_RW_out	: out	STD_LOGIC;		-- lcd read/write	
		SF_D_inout	: inout	STD_LOGIC_VECTOR(11 downto 8));	-- lcd podatkovni signali
		
end lcd_module;

architecture Behavioral of lcd_module is
	
-- KOMPONENTE:	
	
	-- lcd_init komponenta (avtomat, ki inicializira LCD)
	component lcd_init																
		port(
			CLK_in	: in	STD_LOGIC;
			RST_in	: in	STD_LOGIC;
		
			LCD_E_out	: out	STD_LOGIC;
			SF_D_inout	: out	STD_LOGIC_VECTOR(11 downto 8);
			
			INIT_IN_PROG_in	: in	STD_LOGIC;
			INIT_DONE_out	: out	STD_LOGIC);
	end component;
	
	-- lcd_config komponenta (avtomat, ki konfigurira LCD)
	component lcd_config
		port(
			CLK_in	: in	STD_LOGIC;	
			RST_in	: in	STD_LOGIC;	
			
			LCD_E_out	: out	STD_LOGIC;	
			LCD_RS_out	: out	STD_LOGIC;
			LCD_RW_out	: out	STD_LOGIC;
			SF_D_inout	: out	STD_LOGIC_VECTOR(11 downto 8);
			
			CONFIG_IN_PROG_in	: in	STD_LOGIC;
			CONFIG_DONE_out	: out	STD_LOGIC);
	end component;
		
	-- lcd_send komponenta (avtomat, ki LCD-ju poslje podatek)
	component lcd_send
		port(
			CLK_in	: in	STD_LOGIC;
			RST_in	: in	STD_LOGIC;
			
			LCD_E_out	: out	STD_LOGIC;
			LCD_RS_out	: out	STD_LOGIC;
			LCD_RW_out	: out	STD_LOGIC;
			SF_D_inout	: inout	STD_LOGIC_VECTOR(11 downto 8);
			
			DATA_in	:	in STD_LOGIC_VECTOR(9 downto 0);	
			
			SEND_IN_PROG_in	: in	STD_LOGIC;
			SEND_DONE_out	: out	STD_LOGIC);
	end component;
	
-- SIGNALI:

	type STATES is (INIT, CONFIGURE, WAIT_DATA, SEND_DATA, WAIT_FOR_DATA_RDY_RESET, LCD_CLEAR, GO_HOME);	-- definiranje moznih stanj avtomata
	signal currSt, nextSt	:	STATES;		-- trenutno in naslednje stanje
	signal init_in_prog : STD_LOGIC;		-- ali se trenutno odvija inicializacija
	signal init_done : STD_LOGIC;		-- inicializacija koncana
	signal config_in_prog	: STD_LOGIC;		-- ali se trenutno odvija konfiguracija
	signal config_done	: STD_LOGIC;		-- konfiguracija koncana
	signal send_in_prog	: STD_LOGIC;		-- ali se trenutno dovija posiljanje uporabnikovega podatka
	signal send_done	: STD_LOGIC;		-- posiljanje koncano
	
	-- pomozni signali, ki jih priredim dejanskim, ce sem v stanju init
	signal LCD_E_INIT	: STD_LOGIC;
	signal SF_D_INIT	: STD_LOGIC_VECTOR(11 downto 8);					
		
	-- pomozni signali, ki jih priredim dejanskim, ce sem v stanju configure
	signal LCD_E_CONFIG	: STD_LOGIC;
	signal LCD_RS_CONFIG	: STD_LOGIC;
	signal LCD_RW_CONFIG	: STD_LOGIC;
	signal SF_D_CONFIG	: STD_LOGIC_VECTOR(11 downto 8);
	
	-- pomozni signali, ki jih priredim dejanskim, ce sem v stanju send_data
	signal LCD_E_SEND	: STD_LOGIC;
	signal LCD_RS_SEND	: STD_LOGIC;
	signal LCD_RW_SEND	: STD_LOGIC;
	signal SF_D_SEND	: STD_LOGIC_VECTOR(11 downto 8);
	
	-- pomozni signali, ki jih povezem z izhodi
	signal LCD_E	: STD_LOGIC;
	signal LCD_RS	: STD_LOGIC;
	signal LCD_RW	: STD_LOGIC;
	signal SF_D	: STD_LOGIC_VECTOR(11 downto 8);
	signal LCD_READY	: STD_LOGIC;
		
begin

-- IO ASSIGNMENT
	LCD_E_out	<= LCD_E;
	LCD_RS_out	<= LCD_RS;
	LCD_RW_out	<= LCD_RW;
	SF_D_inout	<= SF_D;
	LCD_READY_out	<= LCD_READY;


-- UPORABLJENE KOMPONENTE:

	-- instanca lcd_init komponente
	init_logic:lcd_init
	port map	(
		CLK_in	=> CLK_in,
		RST_in	=> RST_in,
		LCD_E_out	=> LCD_E_INIT,
		SF_D_inout	=>	SF_D_INIT,
		INIT_IN_PROG_in	=> init_in_prog,
		INIT_DONE_out	=> init_done);

	-- instanca lcd_config komponente
	config_logic:lcd_config
	port map(
		CLK_in	=> CLK_in,
		RST_in	=> RST_in,
		LCD_E_out	=> LCD_E_CONFIG,
		LCD_RS_out	=> LCD_RS_CONFIG,
		LCD_RW_out	=> LCD_RW_CONFIG,
		SF_D_inout	=>	SF_D_CONFIG,
		CONFIG_IN_PROG_in	=> config_in_prog,
		CONFIG_DONE_out	=> config_done);
		
	-- instanca lcd_send komponente
	send_logic:lcd_send
	port map(
			CLK_in	=> CLK_in,
			RST_in	=> RST_in,			
			LCD_E_out	=> LCD_E_SEND,
			LCD_RS_out	=> LCD_RS_SEND,
			LCD_RW_out	=> LCD_RW_SEND,
			SF_D_inout	=> SF_D_SEND,			
			DATA_in	=>	DATA_TO_SEND_in,
			SEND_IN_PROG_in	=> send_in_prog,
			SEND_DONE_out	=> send_done);
	
	
-- EO UPORABLJENE KOMPONENTE

-- KONTROLA ENOTA:

	
	-- sinhroni proces
	process (CLK_in)
	begin
		if (rising_edge(CLK_in)) then
			if (RST_in = '1') then 
				currSt <= INIT;
			else
				currSt <= nextSt;
			end if;
		end if;
	end process;
	
	-- asinhroni proces
	process (currSt, init_done, config_done)
	begin
	
		init_in_prog	<= '0';
		config_in_prog	<= '0';
		send_in_prog	<= '0';
		LCD_READY	<= '0';
	
		
		case currSt is
			-- INIT (Power On Initialization)
			when INIT =>		
				
				init_in_prog <= '1';
				
				LCD_RS	<= '0';		-- v init ne rabim
				LCD_RW	<= '0';		-- v init ne rabim
				LCD_E	<= LCD_E_INIT;		-- povezem z INIT avtomatom
				SF_D	<= SF_D_INIT;		-- povezem z INIT avtomatom
				
				if (init_done = '0') then	nextSt <= INIT;
											else	nextSt <= CONFIGURE;
				end if;
				
			-- CONFIGURE (Display Configuration)
			when CONFIGURE =>
				
				config_in_prog	<= '1';
				
				LCD_RS	<= LCD_RS_CONFIG;		-- povezem s CONFIGURE avtomatom
				LCD_RW	<= LCD_RW_CONFIG;		-- povezem s CONFIGURE avtomatom
				LCD_E	<= LCD_E_CONFIG;		-- povezem s CONFIGURE avtomatom
				SF_D	<= SF_D_CONFIG;		-- povezem s CONFIGURE avtomatom
							
				if (config_done = '0')	then nextSt	<= CONFIGURE;
												else nextSt	<=	WAIT_DATA;
				end if;
			
			
			-- WAIT DATA (Wait for user data)
			when WAIT_DATA	=>
				
				LCD_RS	<= '0';
				LCD_RW	<= '0';
				LCD_E	<= '0';
				SF_D	<= "0000";
				
				LCD_READY	<= '1';
				
				if (DATA_RDY_in = '0') then
					nextSt	<= WAIT_DATA;
				else
					nextSt	<= SEND_DATA;
					LCD_READY <= '0';
				end if;
					
			-- SEND DATA (Send user data)
			when SEND_DATA	=>
				
				send_in_prog	<= '1';
							
				LCD_E	<= LCD_E_SEND;		-- povezem s SEND avtomatom
				LCD_RS	<= LCD_RS_SEND;		-- povezem s SEND avtomatom
				LCD_RW	<= LCD_RW_SEND;		-- povezem s SEND avtomatom
				SF_D	<= SF_D_SEND;		-- povezem s SEND avtomatom
												
				if (send_done = '0') then 
					nextSt	<= SEND_DATA;
				else 
					nextSt	<= WAIT_FOR_DATA_RDY_RESET;
				end if;
				
			when WAIT_FOR_DATA_RDY_RESET =>
				send_in_prog	<= '0';
				
				LCD_RS	<= '0';
				LCD_RW	<= '0';
				LCD_E	<= '0';
				SF_D	<= "0000";
				
				LCD_READY	<= '1';
				
				if (DATA_RDY_in = '1') then
					nextSt	<= WAIT_FOR_DATA_RDY_RESET;
				else
					nextSt	<= WAIT_DATA;
				end if;
				
				
			-- others
			when others =>
					
				LCD_RS	<= '0';
				LCD_RW	<= '0';
				LCD_E	<= '0';
				SF_D	<= "0000";
				
				nextSt <= WAIT_DATA;
		end case;
	end process;
	
	
end Behavioral;

