----------------------------------------------------------------------------------
-- Company: FRI
-- Engineer: Miha Moskon
-- 
-- Create Date:    10:22:30 10/04/2007 
-- Design Name: 
-- Module Name:    lcd_send_nibble - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity lcd_send_nibble is

port(
		CLK_in	: in	STD_LOGIC;		-- ura
		RST_in	: in	STD_LOGIC;		-- skupni reset signal
		
		LCD_E_out	: out	STD_LOGIC;		-- lcd enable 
		LCD_RS_out	: out	STD_LOGIC;	-- lcd reg. select
		LCD_RW_out	: out	STD_LOGIC;	-- lcd read/write
		SF_D_inout	: inout	STD_LOGIC_VECTOR(11 downto 8);	-- podatkovni signali LCD-ja
		
		DATA_NIBBLE_in	:	in STD_LOGIC_VECTOR(5 downto 0);		-- ukaz/podatek, ki ga posiljamo LCD-ju
		
		NIBBLE_SEND_IN_PROG_in	: in	STD_LOGIC;	-- odvija se posiljanje
		NIBBLE_SEND_DONE_out	: out	STD_LOGIC);		-- posiljanje je koncano

end lcd_send_nibble;

architecture Behavioral of lcd_send_nibble is
	
	component counter
		generic(
			size : integer := 8);
		port(
			CLK_in	: IN STD_LOGIC;
			WE_in	: IN STD_LOGIC;
			VALUE_in	: IN STD_LOGIC_VECTOR(size-1 downto 0);
			COUNT_UP_out	: OUT STD_LOGIC);		
	end component;

	-- stanja avtomata za posiljanje dela podatka
	type STATES is (DUMMY, INIT1, SEND1, INIT2, SEND2, INIT3, SEND3, DONE);		-- definiranje moznih stanj avtomata: posiljanje je sestavljeno iz treh delov
	signal currSt, nextSt	:	STATES;		-- trenutno in naslednje stanje
	
	-- signali za stevec
	signal WE_counter, count_up	: STD_LOGIC;
	signal value_to_counter	: STD_LOGIC_VECTOR(7 downto 0);
	
	-- signali, ki jih povezemo z izhodi
	
	signal LCD_E	: STD_LOGIC;		-- lcd enable 
	signal LCD_RS	: STD_LOGIC;	-- lcd reg. select
	signal LCD_RW	: STD_LOGIC;	-- lcd read/write
	signal SF_D	: STD_LOGIC_VECTOR(11 downto 8);	-- podatkovni signali LCD-ja	
	signal NIBBLE_SEND_DONE	: STD_LOGIC;		-- posiljanje je koncano

begin

	-- IO ASSIGNMENT
	LCD_E_out	<= LCD_E;
	LCD_RS_out	<= LCD_RS;
	LCD_RW_out	<= LCD_RW;
	SF_D_inout	<= SF_D;
	NIBBLE_SEND_DONE_out	<= NIBBLE_SEND_DONE;	
	
	--------------------------
	-- INSTANCIRANJE KOMPONENT
	--------------------------
	
	-- instanca stevca
	counter8bit:counter
		generic map
		( 
			size => 8
		)
		port map
		(
			CLK_in	=> CLK_in,
			WE_in	=> WE_counter,
			VALUE_in	=> value_to_counter,
			COUNT_UP_out	=> count_up		
		);
		
	
	
	-------------------------------
	-- AVTOMAT ZA POSILJANJE NIBBLA 
	-------------------------------

	sync_proc:process (CLK_in)
	begin
		if (rising_edge(CLK_in)) then
			if ( (RST_in = '1') OR (NIBBLE_SEND_IN_PROG_in = '0') ) then 
				currSt	<= DUMMY;
			else
				currSt	<= nextSt;
			end if;
		end if;
	end process;
	
	send_proc: process (currSt, NIBBLE_SEND_IN_PROG_in, count_up)
	begin
		
		-- lcd		
		LCD_E	<= '0';
		LCD_RS	<= DATA_NIBBLE_in(5);		-- podatki so lahko aktivni kar cel cas posiljanja
		LCD_RW	<= DATA_NIBBLE_in(4);		-- podatki so lahko aktivni kar cel cas posiljanja
		SF_D	<= DATA_NIBBLE_in(3 downto 0);		-- podatki so lahko aktivni kar cel cas posiljanja
		
		-- stevec
		WE_counter <= '0';
		value_to_counter <= (others => '0');
		
		-- izhod iz nibbla, ki lcd_send modulu pove, da je prenos koncan
		NIBBLE_SEND_DONE <= '0';	-- vedno je 0 razen v stanju DONE
		
		case currSt is
			
			-- cakanje na pripravljen podatek
			when DUMMY	=>
				if (NIBBLE_SEND_IN_PROG_in = '0') then
					nextSt	<= DUMMY;
				else
					nextSt	<= INIT1;
				end if;
					
			-- inicializacija prvega dela posiljanja
			when INIT1	=>
				WE_counter	<= '1';
				value_to_counter	<= x"02";	-- 2 u.p. (40 ns)
		
				nextSt	<= SEND1;
				
			-- prvi del posiljanja: podatki so pripravljeni, LCD_E = 0
			when SEND1	=>
				if (count_up = '0') then
					nextSt	<= SEND1;
				else
					nextSt	<= INIT2;
				end if;
				
			-- inicializacija drugega dela posiljanja
			when INIT2	=>
				WE_counter	<= '1';
				value_to_counter	<= x"11";	-- 17 u.p. (330 ns)
				
				nextSt	<= SEND2;
				
			-- drugi del posiljanja: podatki so pripravljeni, LCD_E = 1
			when SEND2	=>
				LCD_E	<= '1';
				
				if (count_up = '0') then
					nextSt	<=	SEND2;
				else
					nextSt	<= INIT3;
				end if;
					
			-- inicializacija tretjega dela posiljanja
			when INIT3	=>
				WE_counter	<= '1';
				value_to_counter	<= x"01";	-- 1 u.p. (>=10 ns)
				
				nextSt	<= SEND3;
				
			-- tretji del posiljanja: podatki so pripravljeni, LCD_E = 0
			when SEND3	=>
				if (count_up = '0') then
					nextSt	<= SEND3;
				else
					nextSt	<= DONE;
				end if;
				
			-- posiljanje nibbla opravljeno
			when DONE	=>
				NIBBLE_SEND_DONE	<= '1';
				
				nextSt	<= DUMMY;

		end case;
	
	end process;

end Behavioral;

